"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.register = void 0;
const shared = require("@volar/shared");
const lsTypes = ['script', 'template'];
function register(ts, { vueDocuments, templateTsLsRaw, scriptTsLsRaw, templateTsHost, scriptTsHost, vueHost }) {
    return {
        getRootFileNames,
        emit,
        getSyntacticDiagnostics,
        getSemanticDiagnostics,
        getGlobalDiagnostics,
        getBindAndCheckDiagnostics,
    };
    function getRootFileNames() {
        var _a, _b, _c, _d;
        const set = new Set([
            ...(_b = (_a = getProgram('script')) === null || _a === void 0 ? void 0 : _a.getRootFileNames().filter(fileName => { var _a; return (_a = scriptTsHost.fileExists) === null || _a === void 0 ? void 0 : _a.call(scriptTsHost, fileName); })) !== null && _b !== void 0 ? _b : [],
            ...(_d = (_c = getProgram('template')) === null || _c === void 0 ? void 0 : _c.getRootFileNames().filter(fileName => { var _a; return (_a = templateTsHost === null || templateTsHost === void 0 ? void 0 : templateTsHost.fileExists) === null || _a === void 0 ? void 0 : _a.call(templateTsHost, fileName); })) !== null && _d !== void 0 ? _d : [],
        ]);
        return [...set.values()];
    }
    // for vue-tsc --noEmit --watch
    function getBindAndCheckDiagnostics(sourceFile, cancellationToken) {
        return getSourceFileDiagnosticsWorker(sourceFile, cancellationToken, 'getBindAndCheckDiagnostics');
    }
    // for vue-tsc --noEmit
    function getSyntacticDiagnostics(sourceFile, cancellationToken) {
        return getSourceFileDiagnosticsWorker(sourceFile, cancellationToken, 'getSyntacticDiagnostics');
    }
    function getSemanticDiagnostics(sourceFile, cancellationToken) {
        return getSourceFileDiagnosticsWorker(sourceFile, cancellationToken, 'getSemanticDiagnostics');
    }
    function getSourceFileDiagnosticsWorker(sourceFile, cancellationToken, api) {
        var _a, _b, _c;
        if (sourceFile) {
            const sourceMap = vueDocuments.fromEmbeddedDocumentUri('script', shared.fsPathToUri(sourceFile.fileName));
            const vueDocument = sourceMap ? vueDocuments.get(sourceMap.sourceDocument.uri) : undefined;
            if (vueDocument) {
                let results = [];
                const sourceMaps = vueDocument.getSourceMaps();
                for (const sourceMap of sourceMaps) {
                    if (sourceMap.lsType === 'nonTs' || !sourceMap.capabilities.diagnostics)
                        continue;
                    const program = getProgram(sourceMap.lsType);
                    const embeddedSourceFile = program === null || program === void 0 ? void 0 : program.getSourceFile(shared.uriToFsPath(sourceMap.mappedDocument.uri));
                    if (embeddedSourceFile) {
                        const errors = transformDiagnostics(sourceMap.lsType, (_a = program === null || program === void 0 ? void 0 : program[api](embeddedSourceFile, cancellationToken)) !== null && _a !== void 0 ? _a : []);
                        results = results.concat(errors);
                    }
                }
                return results;
            }
            else {
                return (_c = (_b = getProgram('script')) === null || _b === void 0 ? void 0 : _b[api](sourceFile, cancellationToken)) !== null && _c !== void 0 ? _c : [];
            }
        }
        return lsTypes.map(lsType => { var _a, _b; return transformDiagnostics(lsType, (_b = (_a = getProgram(lsType)) === null || _a === void 0 ? void 0 : _a[api](sourceFile, cancellationToken)) !== null && _b !== void 0 ? _b : []); }).flat();
    }
    function getGlobalDiagnostics(cancellationToken) {
        return lsTypes.map(lsType => { var _a, _b; return transformDiagnostics(lsType, (_b = (_a = getProgram(lsType)) === null || _a === void 0 ? void 0 : _a.getGlobalDiagnostics(cancellationToken)) !== null && _b !== void 0 ? _b : []); }).flat();
    }
    function emit(targetSourceFile, _writeFile, cancellationToken, emitOnlyDtsFiles, customTransformers) {
        var _a, _b, _c;
        const scriptResult = getProgram('script').emit(targetSourceFile, ((_a = vueHost.writeFile) !== null && _a !== void 0 ? _a : ts.sys.writeFile), cancellationToken, emitOnlyDtsFiles, customTransformers);
        const templateResult = (_b = getProgram('template')) === null || _b === void 0 ? void 0 : _b.emit(targetSourceFile, undefined, cancellationToken, emitOnlyDtsFiles, customTransformers);
        return {
            emitSkipped: scriptResult.emitSkipped,
            emittedFiles: scriptResult.emittedFiles,
            diagnostics: [
                ...transformDiagnostics('script', scriptResult.diagnostics),
                ...transformDiagnostics('template', (_c = templateResult === null || templateResult === void 0 ? void 0 : templateResult.diagnostics) !== null && _c !== void 0 ? _c : []),
            ],
        };
    }
    function getProgram(lsType) {
        var _a;
        return (_a = (lsType === 'script' ? scriptTsLsRaw : templateTsLsRaw)) === null || _a === void 0 ? void 0 : _a.getProgram();
    }
    // transform
    function transformDiagnostics(lsType, diagnostics) {
        var _a, _b;
        const result = [];
        for (const diagnostic of diagnostics) {
            if (diagnostic.file !== undefined
                && diagnostic.start !== undefined
                && diagnostic.length !== undefined) {
                const fileName = shared.normalizeFileName(diagnostic.file.fileName);
                for (const tsOrVueLoc of vueDocuments.fromEmbeddedLocation(lsType, shared.fsPathToUri(fileName), diagnostic.start, diagnostic.start + diagnostic.length, data => !!data.capabilities.diagnostic)) {
                    if (!((_a = vueHost.fileExists) === null || _a === void 0 ? void 0 : _a.call(vueHost, shared.uriToFsPath(tsOrVueLoc.uri))))
                        continue;
                    if (tsOrVueLoc.type === 'source-ts' && lsType !== 'script')
                        continue;
                    let file = shared.uriToFsPath(tsOrVueLoc.uri) === fileName
                        ? diagnostic.file
                        : undefined;
                    if (!file) {
                        let docText = (_b = tsOrVueLoc.sourceMap) === null || _b === void 0 ? void 0 : _b.sourceDocument.getText();
                        if (docText === undefined) {
                            const snapshot = vueHost.getScriptSnapshot(shared.uriToFsPath(tsOrVueLoc.uri));
                            if (snapshot) {
                                docText = snapshot.getText(0, snapshot.getLength());
                            }
                        }
                        if (docText !== undefined) {
                            file = ts.createSourceFile(shared.uriToFsPath(tsOrVueLoc.uri), docText, tsOrVueLoc.uri.endsWith('.vue') ? ts.ScriptTarget.JSON : ts.ScriptTarget.Latest);
                        }
                    }
                    const newDiagnostic = {
                        ...diagnostic,
                        file,
                        start: tsOrVueLoc.range.start,
                        length: tsOrVueLoc.range.end - tsOrVueLoc.range.start,
                    };
                    const relatedInformation = diagnostic.relatedInformation;
                    if (relatedInformation) {
                        newDiagnostic.relatedInformation = transformDiagnostics(lsType, relatedInformation);
                    }
                    result.push(newDiagnostic);
                }
            }
            else if (diagnostic.file === undefined) {
                result.push(diagnostic);
            }
        }
        return result;
    }
}
exports.register = register;
//# sourceMappingURL=apis.js.map